using HackathonPreparing.ApiService.WeatherForecastFeature.EfCore;
using Microsoft.EntityFrameworkCore;

namespace HackathonPreparing.ApiService.WeatherForecastFeature;

public static class WeatherForecastEndpointsExtensions
{
    public static void AddWeatherForecastEndpoints (this WebApplication app)
    {   
        var group = app.MapGroup("/weatherforecast");
        group.MapGet("/", async (WeatherForecastContext db) => await db.Forecasts.ToListAsync());
        group.MapDelete("/{id}", async (WeatherForecastContext db, int id) =>
        {
            var forecast = await db.Forecasts.FindAsync(id);
            if (forecast == null)
            {
                return Results.NotFound();
            }

            db.Forecasts.Remove(forecast);
            await db.SaveChangesAsync();

            return Results.NoContent();
        }).RequireAuthorization("moderator");
        group.MapPost("/", async (WeatherForecastContext db, ApiService.WeatherForecastFeature.WeatherForecast forecast) =>
        {
            db.Forecasts.Add(forecast);
            await db.SaveChangesAsync();

            return Results.Created($"/weatherforecast/{forecast.Id}", forecast);
        });
        group.MapPut("/{id}", async (WeatherForecastContext db, int id, ApiService.WeatherForecastFeature.WeatherForecast forecast) =>
        {
            if (id != forecast.Id)
            {
                return Results.BadRequest();
            }

            db.Entry(forecast).State = EntityState.Modified;

            try
            {
                await db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (await db.Forecasts.FindAsync(id) == null)
                {
                    return Results.NotFound();
                }

                throw;
            }

            return Results.NoContent();
        }).RequireAuthorization("moderator");
        group.WithTags("weatherforecast");
    }
}